@@ -19,12 +19,17 @@
 
 from __future__ import print_function
 
+import glob
 import os
 import subprocess
 import sys
+import syslog
+
+import gettext
 
 from ubiquity import i18n, misc, osextras, plugin, upower
-from ubiquity.install_misc import archdetect, is_secure_boot
+from ubiquity.install_misc import (archdetect, is_secure_boot,
+                                   minimal_install_rlist_path)
 
 NAME = 'prepare'
 AFTER = 'wireless'
@@ -38,16 +43,25 @@ OEM = False
 # need the dbfilter for that get.
 
 class PreparePageBase(plugin.PluginUI):
-    plugin_title = 'ubiquity/text/prepare_heading_label'
+    plugin_title = 'mint:Multimedia codecs'
+    download_updates = True
+    download_updates_enabled = True
 
     def __init__(self, *args, **kwargs):
         plugin.PluginUI.__init__(self)
 
     def plugin_set_online_state(self, state):
+        # if we're disabling, remember what it was before so we can re-enable
+        # properly if network comes back
+        if self.prepare_network_connection.get_state() and not state:
+            self.download_updates = self.get_download_updates()
         self.prepare_network_connection.set_state(state)
         self.enable_download_updates(state)
+        self.download_updates_enabled = state
         if not state:
             self.set_download_updates(False)
+        else:
+            self.set_download_updates(self.download_updates)
 
     def set_sufficient_space(self, state, required, free):
         if not state:
@@ -58,6 +72,7 @@ class PreparePageBase(plugin.PluginUI):
         self.prepare_sufficient_space.set_state(state)
 
     def plugin_translate(self, lang):
+        self.rst_title_text = i18n.get_string('rst_header', lang)
         return
 
 
@@ -104,11 +119,21 @@ class PageGtk(PreparePageBase):
         self.secureboot_box.set_sensitive(False)
         self.password_grid.set_sensitive(False)
 
+        self.minimal_install_vbox.set_visible(
+            os.path.exists(minimal_install_rlist_path))
+
         self.prepare_page = builder.get_object('stepPrepare')
         self.insufficient_space_page = builder.get_object('stepNoSpace')
+        self.rst_page = builder.get_object('stepRST')
+        self.rst_label = builder.get_object('label_using_rst')
+        self.rst_label.connect('activate-link', self.on_link_clicked)
         self.current_page = self.prepare_page
         self.plugin_widgets = self.prepare_page
-        self.plugin_optional_widgets = [self.insufficient_space_page]
+        self.plugin_optional_widgets = [self.insufficient_space_page,
+                                        self.rst_page]
+
+    def on_link_clicked(self, widget, uri):
+        misc.launch_uri(uri)
 
     def plugin_get_current_page(self):
         return self.current_page
@@ -121,6 +146,22 @@ class PageGtk(PreparePageBase):
 
         self.controller.go_to_page(self.current_page)
 
+    def show_rst_page(self):
+        for page in self.controller._wizard.pages:
+            if page.module.NAME == NAME:
+                page.title = 'ubiquity/text/rst_header'
+                break
+        self.current_page = self.rst_page
+        self.controller.go_to_page(self.current_page)
+        return True
+
+    def plugin_on_next_clicked(self):
+        if self.current_page != self.rst_page:
+            return
+
+        self.controller._wizard.do_reboot()
+        return True
+
     def set_using_secureboot(self, secureboot):
         self.using_secureboot = secureboot
         self.secureboot_box.set_visible(secureboot)
@@ -143,6 +184,12 @@ class PageGtk(PreparePageBase):
     def get_download_updates(self):
         return self.prepare_download_updates.get_active()
 
+    def set_minimal_install(self, val):
+        self.prepare_minimal_install.set_active(val)
+
+    def get_minimal_install(self):
+        return self.prepare_minimal_install.get_active()
+
     def set_allow_nonfree(self, allow):
         if not allow:
             self.prepare_nonfree_software.set_active(False)
@@ -165,6 +212,9 @@ class PageGtk(PreparePageBase):
         PreparePageBase.plugin_translate(self, lang)
         release = misc.get_release()
 
+        self.prepare_nonfree_software.set_label(gettext.dgettext("mintreport", "Install multimedia codecs"))
+        self.prepare_foss_disclaimer.set_label(gettext.dgettext("mintreport", "Multimedia codecs are required to play some video formats and to properly render some websites."))
+
         from gi.repository import Gtk
         for widget in [self.prepare_download_updates,
                        self.label_required_space,
@@ -255,6 +305,8 @@ class PageKde(PreparePageBase):
             from PyQt5 import uic
             from PyQt5 import QtGui
             self.page = uic.loadUi('/usr/share/ubiquity/qt/stepPrepare.ui')
+            self.prepare_minimal_install = self.page.prepare_minimal_install
+            self.qt_label_minimal_install = self.page.qt_label_minimal_install
             self.prepare_download_updates = self.page.prepare_download_updates
             self.prepare_nonfree_software = self.page.prepare_nonfree_software
             self.prepare_foss_disclaimer = self.page.prepare_foss_disclaimer
@@ -278,6 +330,9 @@ class PageKde(PreparePageBase):
             except Exception as e:
                 # TODO use an inconsistent state?
                 print('unable to set up power source watch:', e)
+            if not os.path.exists(minimal_install_rlist_path):
+                self.qt_label_minimal_install.hide()
+                self.prepare_minimal_install.hide()
             try:
                 self.prepare_network_connection = StateBox(self.page)
             except Exception as e:
@@ -289,6 +344,9 @@ class PageKde(PreparePageBase):
         self.set_using_secureboot(False)
         self.plugin_widgets = self.page
 
+    def show_rst_page(self):
+        return False
+
     def show_insufficient_space_page(self, required, free):
         from PyQt5 import QtWidgets
         QtWidgets.QMessageBox.critical(self.page,
@@ -337,6 +395,14 @@ class PageKde(PreparePageBase):
         from PyQt5.QtCore import Qt
         return self.prepare_download_updates.checkState() == Qt.Checked
 
+    def set_minimal_install(self, val):
+        self.prepare_minimal_install.setChecked(val)
+
+    def get_minimal_install(self):
+        if self.prepare_minimal_install.isChecked():
+            return True
+        return False
+
     def set_allow_nonfree(self, allow):
         if not allow:
             self.prepare_nonfree_software.setChecked(False)
@@ -384,10 +450,32 @@ class Page(plugin.Plugin):
             if is_secure_boot():
                 self.ui.set_using_secureboot(True)
 
-        download_updates = self.db.get('ubiquity/download_updates') == 'true'
-        self.ui.set_download_updates(download_updates)
+        self.ui.download_updates = self.db.get('ubiquity/download_updates') == 'true'
+        if self.ui.download_updates_enabled:
+            self.ui.set_download_updates(self.ui.download_updates)
+        minimal_install = self.db.get('ubiquity/minimal_install') == 'true'
+        self.ui.set_minimal_install(minimal_install)
         self.apply_debconf_branding()
-        self.setup_sufficient_space()
+
+        # wait for it to finish
+        if self.frontend.ubuntu_drivers:
+            self.frontend.ubuntu_drivers.communicate()
+            self.frontend.ubuntu_drivers = None
+
+        # output whether there are OEM packages for this system
+        try:
+            with open('/run/ubuntu-drivers-oem.autoinstall', 'r') as f:
+                syslog.syslog(F'ubuntu-drivers list-oem finished with: "{" ".join(f.read().splitlines())}"')
+        except FileNotFoundError:
+            syslog.syslog('ubuntu-drivers list-oem finished with no available packages')
+
+        if self.should_show_rst_page():
+            if not self.ui.show_rst_page():
+                self.setup_sufficient_space()
+            else:
+                self.ui.plugin_is_restart = True
+        else:
+            self.setup_sufficient_space()
         command = ['/usr/share/ubiquity/simple-plugins', 'prepare']
         questions = ['ubiquity/use_nonfree']
         return command, questions
@@ -398,6 +486,14 @@ class Page(plugin.Plugin):
                          'ubiquity/text/free_space']:
             self.db.subst(template, 'RELEASE', release.name)
 
+    def should_show_rst_page(self):
+        search = '/sys/module/ahci/drivers/pci:ahci/*/remapped_nvme'
+        for remapped_nvme in glob.glob(search):
+            with open(remapped_nvme, 'r') as f:
+                if int(f.read()) > 0:
+                    return True
+        return os.environ.get('SHOW_RST_UI', '0') == '1'
+
     def setup_sufficient_space(self):
         # TODO move into prepare.
         size = misc.install_size()
@@ -425,11 +521,13 @@ class Page(plugin.Plugin):
         return biggest
 
     def ok_handler(self):
-        download_updates = self.ui.get_download_updates()
+        download_updates = False
+        minimal_install = self.ui.get_minimal_install()
         use_nonfree = self.ui.get_use_nonfree()
         secureboot_key = self.ui.get_secureboot_key()
         self.preseed_bool('ubiquity/use_nonfree', use_nonfree)
         self.preseed_bool('ubiquity/download_updates', download_updates)
+        self.preseed_bool('ubiquity/minimal_install', minimal_install)
         if self.ui.using_secureboot and secureboot_key:
             self.preseed('ubiquity/secureboot_key', secureboot_key, seen=True)
         if use_nonfree:
